@@ -4,22 +4,23 @@ import sys
 from typing import TYPE_CHECKING
 
 from PyQt5.QtCore import Qt, QSize
-from PyQt5.QtGui import QGuiApplication
+from PyQt5.QtGui import QGuiApplication, QResizeEvent
 from PyQt5.QtWidgets import QWidget, QHBoxLayout, QLabel, QVBoxLayout, \
     QSizePolicy, QScrollArea, QFrame
 
-from feeluown.excs import ProviderNotFound, ResourceNotFound
+from feeluown.excs import ResourceNotFound
 from feeluown.library import (
-    SupportsSongHotComments, SupportsSongSimilar, SupportsSongWebUrl,
-    NotSupported, ModelFlags
+    SupportsSongHotComments, SupportsSongSimilar, ModelFlags,
 )
 from feeluown.player import Lyric
-from feeluown.models.uri import reverse, resolve
+from feeluown.library import reverse, resolve
 from feeluown.utils import aio
 from feeluown.utils.aio import run_afn
 from feeluown.utils.reader import create_reader
 from feeluown.gui.components import SongMVTextButton
-from feeluown.gui.helpers import BgTransparentMixin, fetch_cover_wrapper, resize_font
+from feeluown.gui.helpers import BgTransparentMixin, fetch_cover_wrapper, \
+    resize_font
+from feeluown.gui.widgets.labels import ElidedLineLabel
 from feeluown.gui.widgets.header import LargeHeader, MidHeader
 from feeluown.gui.widgets.textbtn import TextButton
 from feeluown.gui.widgets.cover_label import CoverLabelV2
@@ -28,6 +29,7 @@ from feeluown.gui.widgets.lyric import LyricView
 from feeluown.gui.widgets.song_minicard_list import (
     SongMiniCardListDelegate, SongMiniCardListModel, SongMiniCardListView
 )
+from .template import render_error_message
 
 if TYPE_CHECKING:
     from feeluown.app.gui_app import GuiApp
@@ -48,12 +50,9 @@ async def render(req, **kwargs):  # pylint: disable=too-many-locals,too-many-bra
     app: GuiApp = req.ctx['app']
     song = req.ctx['model']
 
-    try:
-        provider = app.library.get_or_raise(song.source)
-    except ProviderNotFound as e:
-        err_view = InlineErrorMessageView()
-        err_view.show_msg(f'无法展示歌曲详情：{repr(e)}')
-        app.ui.right_panel.set_body(err_view)
+    provider = app.library.get(song.source)
+    if provider is None:
+        await render_error_message(app, f'没有相应的资源提供方 {song.source}')
         return
 
     # TODO: Initialize the view with song object, and it should reduce
@@ -63,13 +62,14 @@ async def render(req, **kwargs):  # pylint: disable=too-many-locals,too-many-bra
 
     try:
         upgraded_song = await aio.run_fn(app.library.song_upgrade, song)
-    except (NotSupported, ResourceNotFound):
+    except ResourceNotFound:
         upgraded_song = None
     if upgraded_song is not None:
         song = upgraded_song
 
     # Before fetching more data, show some widgets.
-    view.title_label.setText(song.title)
+    view.title_label.set_src_text(song.title)
+    view.title_label.setToolTip(song.title)
     view.play_btn.clicked.connect(lambda: app.playlist.play_model(song))
 
     # Show other widgets.
@@ -80,7 +80,7 @@ async def render(req, **kwargs):  # pylint: disable=too-many-locals,too-many-bra
     if album is not None:
         try:
             upgraded_album = await aio.run_fn(app.library.album_upgrade, song.album)
-        except (NotSupported, ResourceNotFound):
+        except ResourceNotFound:
             upgraded_album = None
     else:
         upgraded_album = None
@@ -152,7 +152,7 @@ class SongWikiLabel(QLabel):
         # Only show the date, like yyyy-mm-dd. Do not show hour/minutes/seconds.
         if song.date:
             date = song.date
-        elif album is not None and ModelFlags.normal in album.meta.flags:
+        elif album is not None and ModelFlags.normal in ModelFlags(album.meta.flags):
             date = album.released
         else:
             date = ''
@@ -188,12 +188,18 @@ class RightCon(QWidget):
         return QSize(130, size_hint.height())
 
 
+class Title(LargeHeader, ElidedLineLabel):
+    pass
+
+
 class SongExploreView(QWidget):
     def __init__(self, app: GuiApp):
         super().__init__(parent=None)
         self._app = app
 
-        self.title_label = LargeHeader()
+        self._title_cover_spacing = 10
+        self._left_right_spacing = 30
+        self.title_label = Title()
         self.similar_songs_header = MidHeader('相似歌曲')
         self.comments_header = MidHeader('热门评论')
         self.lyric_view = LyricView(parent=self)
@@ -242,11 +248,11 @@ class SongExploreView(QWidget):
         self._layout.setContentsMargins(10, 20, 10, 0)
 
         self._layout.addWidget(self._left_con_scrollarea, 3)
-        self._layout.addSpacing(30)
+        self._layout.addSpacing(self._left_right_spacing)
         self._layout.addWidget(self._right_con, 1)
 
         self._left_layout.addLayout(self._left_top_layout)
-        self._left_layout.addSpacing(10)
+        self._left_layout.addSpacing(self._title_cover_spacing)
         self._left_layout.addWidget(self.comments_header)
         self._left_layout.addWidget(self.comments_view)
         self._left_layout.addWidget(self.similar_songs_header)
@@ -271,13 +277,11 @@ class SongExploreView(QWidget):
         self._right_layout.addWidget(self.lyric_view)
 
     async def maybe_show_web_url_btn(self, provider, song):
-        if isinstance(provider, SupportsSongWebUrl):
-            async def copy_song_web_url():
-                web_url = await aio.run_fn(provider.song_get_web_url, song)
-                QGuiApplication.clipboard().setText(web_url)
-                self._app.show_msg(f'已经复制：{web_url}')
-
-            self.copy_web_url_btn.clicked.connect(lambda: aio.run_afn(copy_song_web_url))
+        try:
+            web_url = self._app.library.song_get_web_url(song)
+        except ResourceNotFound:
+            self.copy_web_url_btn.hide()
+        else:
             # TODO: Open url in browser when alt key is pressed. Use
             # QDesktopServices.openUrl to open url in browser, and
             # you may use QGuiApplication::keyboardModifiers to check
@@ -285,8 +289,12 @@ class SongExploreView(QWidget):
             #
             # NOTE(cosven): Since switching from applications is inconvenience,
             # the default behaviour of button is url-copy instead of url-open.
-        else:
-            self.copy_web_url_btn.hide()
+
+            async def copy_song_web_url():
+                QGuiApplication.clipboard().setText(web_url)
+                self._app.show_msg(f'已经复制：{web_url}')
+
+            self.copy_web_url_btn.clicked.connect(lambda: aio.run_afn(copy_song_web_url))
 
     async def show_song_wiki(self, song, album):
         aio.run_afn(self.song_wiki_label.show_song, song, album)
@@ -324,14 +332,10 @@ class SongExploreView(QWidget):
                 self.lyric_view.on_line_changed, weak=True)
             return
 
-        try:
-            lyric_model = self._app.library.song_get_lyric(song)
-        except NotSupported:
-            pass
-        else:
-            if lyric_model is None:
-                return
-            self.lyric_view.set_lyric(Lyric.from_content(lyric_model.content))
+        lyric_model = self._app.library.song_get_lyric(song)
+        if lyric_model is None:
+            return
+        self.lyric_view.set_lyric(Lyric.from_content(lyric_model.content))
 
     async def maybe_show_song_pic(self, song, album):
         if album:
@@ -339,18 +343,13 @@ class SongExploreView(QWidget):
                         album.cover,
                         reverse(album) + '/cover')
         else:
-            if ModelFlags.v2 in song.meta.flags:
-                aio.run_afn(self.cover_label.show_cover,
-                            song.pic_url,
-                            reverse(song) + '/pic_url')
+            aio.run_afn(self.cover_label.show_cover,
+                        song.pic_url,
+                        reverse(song) + '/pic_url')
 
-
-class InlineErrorMessageView(QLabel):
-    """Error message view
-    """
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.setAlignment(Qt.AlignCenter)
-
-    def show_msg(self, msg):
-        self.setText(msg)
+    def resizeEvent(self, e: QResizeEvent) -> None:
+        margins = self.layout().contentsMargins()
+        margin_h = margins.left() + margins.right()
+        self._left_con.setMaximumWidth(
+            self.width() - margin_h - self._left_right_spacing - self._right_con.width())
+        return super().resizeEvent(e)
