@@ -1,10 +1,9 @@
 # flake8: noqa
 from .library import Library
-from .provider import AbstractProvider, dummy_provider
-from .provider_v2 import ProviderV2
+from .provider import AbstractProvider, ProviderV2, Provider
 from .flags import Flags as ProviderFlags
 from .model_state import ModelState
-from .model_protocol import (
+from .model_protocol import (  # deprecated
     BriefSongProtocol,
     BriefVideoProtocol,
     BriefArtistProtocol,
@@ -20,8 +19,13 @@ from .models import ModelFlags, BaseModel, ModelType, SearchType, \
     LyricModel, VideoModel, BriefVideoModel, \
     ArtistModel, AlbumModel, PlaylistModel, BriefPlaylistModel, \
     fmt_artists_names, AlbumType, SimpleSearchResult, \
-    get_modelcls_by_type, \
+    get_modelcls_by_type, MediaFlags, \
     V2SupportedModelTypes
-from .excs import NotSupported, NoUserLoggedIn, ModelNotFound, \
+from .excs import NoUserLoggedIn, ModelNotFound, \
     ProviderAlreadyExists, ResourceNotFound, MediaNotFound
 from .provider_protocol import *
+from .uri import (
+    Resolver, reverse, resolve, ResolverNotFound, ResolveFailed,
+    parse_line, NS_TYPE_MAP,
+)
+from .collection import Collection, CollectionType
